#include <iostream>

#include <boost/timer_new.hpp>

using namespace std;

inline void mysleep(int milliseconds)
{
	cout << "Sleeping for " << (milliseconds/1000.0) << " seconds" << endl;
	#ifdef WIN32
		Sleep(milliseconds);
	#else
		usleep(milliseconds*1000);
	#endif
}

template <class timer>
inline void print_value(const timer& tmr)
{
	cout << "Current value: " << tmr.elapsed().seconds() << "." << tmr.elapsed().fractional_seconds()  << endl;
}

//#define SECOND_TIMER

int main()
{
	cout << "Starting timer" << endl;
#ifdef SECOND_TIMER
	boost::second_timer tmr;
#else
	boost::microsec_timer tmr;
#endif
	mysleep(200);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);
	
	cout << "Stopping timer" << endl;
	tmr.pause();
	print_value(tmr);

	mysleep(1000);
	cout << "This last sleep should not be timed as the timer is stopped" << endl;
	print_value(tmr);

	cout << "Resuming timer" << endl;
	tmr.resume();
	mysleep(200);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);

	cout << "Restarting timer" << endl;
	tmr.reset();
	mysleep(200);
	print_value(tmr);
	mysleep(1500);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);

	return 0;
}
