#ifndef BOOST_TIMER_HPP
#define BOOST_TIMER_HPP

// Copyright (c) 2005 CrystalClear Software, Inc.
// (C) Copyright Vaucher Philippe 2006
// Use, modification and distribution is subject to the 
// Boost Software License, Version 1.0. (See accompanying
// file LICENSE-1.0 or http://www.boost.org/LICENSE-1.0)
// Original draft: Jeff Garland 2005
// Extensions: Vaucher Philippe 2006


// -------------------------- Includes --------------------------

// Boost headers
#include <boost/date_time/posix_time/posix_time.hpp>


// -------------------------- Code --------------------------

namespace boost
{

	template<class clock_type, class time_type = boost::posix_time::ptime, class time_duration_type = boost::posix_time::time_duration>
	class timer
	{
	public:

		enum start_options { AUTO_START, MANUAL_START };

		timer(time_duration_type initial_duration = time_duration_type(0,0,0), start_options start_op = AUTO_START) : m_start(boost::date_time::not_a_date_time), m_elapsed(initial_duration)
		{
			if(start_op == AUTO_START)
				start();
		}

		void start() 
		{
			if(m_start.is_not_a_date_time())
				m_start = clock_type::local_time();
		}

		void restart()
		{
			reset();
			start();
		}

		time_duration_type elapsed() const
		{
			if(!m_start.is_not_a_date_time())
			{
				time_type current(clock_type::local_time());
				m_elapsed += (current - m_start);
				m_start = current;
			}
			return m_elapsed;
		}

		void pause()
		{
			m_elapsed += (clock_type::local_time() - m_start);
			m_start = boost::date_time::not_a_date_time; 
		}

		void resume()
		{
			if(m_start.is_not_a_date_time())
				m_start = clock_type::local_time();
		}

		void reset()
		{
			m_elapsed = time_duration_type(0,0,0);
			if(!m_start.is_not_a_date_time())
				m_start = clock_type::local_time();
		}

	private:

		mutable time_type m_start;
		mutable time_duration_type m_elapsed;

	};

	// Convenience typedefs
	typedef boost::timer<boost::posix_time::second_clock>	second_timer;
	typedef boost::timer<boost::posix_time::microsec_clock> microsec_timer;

}

#endif
